/*
** Module   : ComboBoxGuiImpl.java
** Abstract : GUI combo-box implementation
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 EVL 20150421 Initial version.
** 002 GES 20150508 Eliminated AWT dependencies.
** 003 EVL 20150514 Adding GUI specific keyboard and mouse default exit events.  Turning on
**                  tooltip engine.  Adding consistent cleanup on drop-down exit for GUI mode.
*/

package com.goldencode.p2j.ui.client.gui;

// this usage of AWT events is intentional, they are being used separately from the
// AWT/Swing event model itself; no other AWT/Swing usage is occurring in this class
import java.awt.event.MouseEvent;
import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.gui.GuiFontResolver.FontCache;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.ui.client.ScrollBar.Bars;
import com.goldencode.p2j.util.*;
import com.goldencode.util.*;

/**
 * Specific GUI implementation of COMBO-BOX widget.
 */
@SuppressWarnings("unchecked")
public class ComboBoxGuiImpl
extends ComboBox<GuiOutputManager>
{
   /** Shift in X to draw text inside entry. */
   public static final int TEXT_SHIFT = 4;
   
   /** Shift in X and Y to draw entry field for DROP-DOWN and SIMPLE modes. */
   public static final int ENTRY_SHIFT = 2;
   
   /** Dimension delta to draw entry field for DROP-DOWN and SIMPLE modes. */
   public static final int ENTRY_DELTA = ENTRY_SHIFT + 1;
   
   /** Screen driver */
   private GuiDriver gd;

   /** Output manager */
   private GuiOutputManager screen;
   
   /** Coordinate converter */
   private CoordinatesConversion cc;

   /** Color resolver */
   private GuiColorResolver gc;
   
   /** Font resolver */
   private GuiFontResolver gf;
      
   /** Selected item box color. */
   private ColorRgb selectionBack;

   /** Selected dotted line color. */
   private ColorRgb selectionDots;

   /** Light shadow box color. */
   private ColorRgb lightShadow;

   /** Dark shadow box color. */
   private ColorRgb darkShadow;
   
   /** Very dark shadow box color. */
   private ColorRgb veryDarkShadow;
   
   /** Pressed button state flag. */
   private boolean pressed = false;
   
   /** Image to draw drop down button. */
   private ImageGuiImpl imgButt = null; 
   
   /** Image to draw drop down button in disabled state. */
   private ImageGuiImpl imgButtDisabled = null;
   
   /** Editable entry field for SIMPLE and DROP-DOWN modes. */
   private EntryFieldGuiImpl entryField = null;
   
   /** The list for SIMPLE combo-box mode. */
   private ScrollableSelectionListGuiImpl simpleList = null;
   
   /** The list for SIMPLE combo-box mode. */
   private ScrollPaneGuiImpl simpleListPane = null;
   
   /** Name of the image to use for drop-down list activation button. */
   private static final String DROP_DOWN_BUTTON_NAME = "btn-down-arrow";
   
   /** Name of the image to use for drop-down list activation button disabled sate. */
   private static final String DROP_DOWN_BUTTON_NAME_INS = "btn-down-arrow-ins";
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public ComboBoxGuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (ComboBoxConfig) cfg);
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    wcfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, ComboBoxConfig wcfg)
   {
      super.initialize(id, wcfg);
      
      // screen driver
      gd = (GuiDriver) OutputManager.getDriver();
      
      // output manager
      screen = screen();
      cc = screen.coordinates();
      
      // font resolver
      gf = new GuiFontResolver(Window.resolveWindow(this), gd, config(), false);
      
      // initialize the shadow colors
      lightShadow    = gd.getSysColor(SysColor.COLOR_WINDOW);
      darkShadow     = gd.getSysColor(SysColor.COLOR_3DSHADOW);
      veryDarkShadow = gd.getSysColor(SysColor.COLOR_3DDKSHADOW);
      selectionBack  = gd.getSysColor(SysColor.COLOR_HIGHLIGHT);
      
      // the color below is negative to selectionBack
      selectionDots  = new ColorRgb(255 - selectionBack.getRed(),
                                    255 - selectionBack.getGreen(),
                                    255 - selectionBack.getBlue());

      // create required images to draw drop-down button
      if (config().mode == ComboBoxConfig.Mode.DROP_DOWN ||
          config().mode == ComboBoxConfig.Mode.DROP_DOWN_LIST)
      {
         imgButt = initDropDownButton(DROP_DOWN_BUTTON_NAME);
         imgButtDisabled = initDropDownButton(DROP_DOWN_BUTTON_NAME_INS);
      }
      
      // register tooltip action listener
      gd.registerTooltipWorker(this, gf.font().font, ToolTip.isActive(config().tooltip));
   }

   /**
    * Draw the combo-box. Called by this ComboBox's parent container.
    */
   @Override
   public void draw()
   {
      // initialize entry field for SIMPLE and DROP-DOWN modes but only on the first draw
      if ((config().mode == ComboBoxConfig.Mode.SIMPLE ||
           config().mode == ComboBoxConfig.Mode.DROP_DOWN) && entryField == null)
      {
         // create new fill-in
         createEntryField();
         
         // and set up initial value for entry field
         entryField.setValue(new character(model().selected()));
      }
      
      // for simple mode we need to create permanent drop-down here
      if (config().mode == ComboBoxConfig.Mode.SIMPLE && simpleListPane == null)
      {
         createSimpleList();
      }
      
      // combo-box options need to be refreshed before every painting
      final ComboBoxConfig cbbcfg = config();
      gc = new GuiColorResolver(gd, cbbcfg, SysColor.COLOR_MENU, SysColor.COLOR_BTNTEXT);
      
      // determine colors to use
      gc.colors();
      
      // resolve font
      final String text = getLabelText();
      final FontCache fntCache = gf.font();

      // get location and size
      NativePoint p = physicalLocation();
      final NativeDimension d = physicalDimension();

      // set up clipping rectangle
      NativeRectangle clip = simpleListPane != null
         ? clipRectangle(p, new NativeDimension(d.width + 3, d.height +
                                                simpleListPane.physicalDimension().height + 6))
         : clipRectangle(p, new NativeDimension(d.width, d.height));
      
      // clippable drawing start 
      gd.draw(p, clip, new Runnable()
      {

         @Override
         public void run()
         {
            // inside this call widget origin is always (0,0)
            NativePoint po = new NativePoint(0, 0);

            boolean focused = hasFocus() && isEnabled();

            // the field border            
            drawFieldBorder(po, d);

            // text in DROP-DOWN-LIST mode
            if (config().mode == ComboBoxConfig.Mode.DROP_DOWN_LIST)
            {
               drawEntryField(po, d, fntCache.font, focused);
            }
            // in SIMPLE or DROP-DOWN modes the entry field drawing
            // is happening from the frame the entry field belongs to

            // and drop-down button for modes except simple
            if (config().mode != ComboBoxConfig.Mode.SIMPLE)
            {
               drawButton(po, d, pressed);
            }

            // draw drop-down in SIMPLE mode border
            if (config().mode == ComboBoxConfig.Mode.SIMPLE && simpleListPane != null)
            {
               NativePoint slP = new NativePoint(0, d.height + 2);
               NativeDimension slDim = simpleListPane.physicalDimension();
               slDim.width += 3;
               slDim.height += 3;
               drawFieldBorder(slP, slDim);
            }
            // in SIMPLE mode the list drawing is happening from the frame the list belongs to
         }
         
      });
      
   }

   
   /**
    * Sets position of terminal's cursor to the first char of the combo-box.
    */
   @Override
   public void drawCaret()
   {
      // this check is required since the method can be called from multiple
      // drivers
      
      // nothing to do in GUI
   }

   /**
    * Obtain the input focus.
    */
   @Override
   public void requestFocus()
   {
      if (isEnabled())
      {
         // generate the FOCUS_GAINED event
         if (entryField != null)
         {
            entryField.requestFocus();
         }
         else
         {
            super.requestFocus();
         }
      }
   }

   /**
    * Notification of a mouse clicked event occurred for this widget.  Opening drop-down list.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseClicked(MouseEvent e)
   {
      // open drop-down if not already opened
      if (isEnabled() && !dropDownActive && config.mode != ComboBoxConfig.Mode.SIMPLE)
      {
         // activate drop-down
         activate();
      }
   }

   /**
    * Notification of a mouse pressed event occurred for this widget.  Redraw combo-box in pressed
    * state and request focus.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mousePressed(MouseEvent e)
   {
      // push button effect
      if (isEnabled() && !pressed)
      {
         requestFocus();
         pressed = true;
         repaint();
      }
   }

   /**
    * Notification of a mouse released event occurred for this widget.  Redraw combo-box in
    * released state.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseReleased(MouseEvent e)
   {
      // release button effect
      if (isEnabled() && pressed)
      {
         pressed = false;
         repaint();
      }
   }

   /**
    * Process key events that have been generated by this component.
    * 
    * @param    keyEvent
    *           The key event to be handled.
    */
   @Override
   public void processKeyEvent(KeyInput keyEvent)
   {
      if (!isEnabled())
      {
         return;
      }
      
      if (!dropDownActive)
      {
         int key = keyEvent.actionCode();
         
         if (entryField != null && key != Keyboard.SE_VALUE_CHANGED)
         {
            entryField.processKeyEvent(keyEvent);
         }
         else
         {
            int newIndx = ScrollableList.getFirstCharSelection(keyEvent, model(),
                                                         model().getSelectedIndex());
            if (newIndx >= 0)
            {
               // select the new item
               setNewSelection(newIndx);
               // and consume event
               keyEvent.consume();
            }
            else
            {
               super.processKeyEvent(keyEvent);
            }
         }
      }
      else
      {
         super.processKeyEvent(keyEvent);
      }
   }
   
   /**
    * Process the action associated with the given key in an independent drawing bracket.  In
    * GUI combo-box the precessing is highly dependend on the current mode.
    * <p>
    * This must be called only in the context of widget-level event processing.
    * 
    * @param    keyEvent
    *           The key to be processed.
    */
   @Override
   public void processSystemKey(final KeyInput keyEvent)
   {
      // get the key code
      int key = keyEvent.actionCode();
   
      // if the key is neither UP nor DOWN - call superclass for any mode
      if (key != Keyboard.KA_CURSOR_DOWN && key != Keyboard.KA_CURSOR_UP)
      {
         super.processSystemKey(keyEvent);
      }
      // below is the UP, DOWN keys processing depending on mode
      else if (config.mode == ComboBoxConfig.Mode.SIMPLE)
      {
         // the mode is simple - route processing to the list for both keys
         simpleList.processKeyEvent(keyEvent);
      }
      // the mode is DROP-DOWN or DROP-DOWN-LIST
      else if (key == Keyboard.KA_CURSOR_DOWN)
      {
         // select the next item
         setNewSelection(model().getSelectedIndex() + 1);
      }
      else if (key == Keyboard.KA_CURSOR_UP)
      {
         // select the previous item
         setNewSelection(model().getSelectedIndex() - 1);
      }
   }

   /**
    * Calculate width limit.  In GUI we need to adjust the width for different font metrics.
    * 
    * @param   body
    *          Perform calculation for body part of widget.
    * 
    * @return  Calculated width limit.
    */
   @Override
   public double calcLimit(boolean body)
   {
      // this is the ChUI native limit
      double txtLimit = super.calcLimit(body);
      
      // calculate the GUI version
      // TODO: identify exactly how to compute this difference without hardcoding
      if (config().mode == ComboBoxConfig.Mode.SIMPLE)
      {
         txtLimit += 4.0;
      }
      else
      {
         txtLimit += 6.0;
      }
      
      return txtLimit;
   }

   /**
    * Get widget screen value.
    * 
    * @return  Current widget screen value.
    */
   @Override
   public BaseDataType getScreenValue()
   {
      return entryField != null ? new character(entryField.getScreenValue(true))
                                : super.getScreenValue();
   }

   /**
    * Shows or hides the combo-box and its label.
    *
    * @param    visible_
    *           <code>true</code> to make the widget visible, 
    *           <code>false</code> to hide the widget.
    */
   @Override
   public void setVisible(boolean visible_)
   {
      super.setVisible(visible_);
   }
   
   /**
    * Get widget physical dimension, expressed in units native to UI type.
    *  
    * @return  widget dimension.
    */
   @Override
   public NativeDimension physicalDimension()
   {
      // get combo-box generic dimension
      NativeDimension dim = super.physicalDimension();
      
      return dim;
   }

   /**
    * This method is invoked when selection is changed.
    */
   @Override
   public void changed()
   {
      // call common code first
      super.changed();

      // then GUI specific
      if (config().mode == ComboBoxConfig.Mode.SIMPLE && simpleList != null)
      {
         setNewSelection(simpleList.index());
      }
      else if (dropDownActive && dropDown != null)
      {
         setNewSelection(dropDown.getSelectedIndex());
      }
   }

   /**
    * Enable/disable widget.
    * 
    * @param   enabled
    *          <code>TRUE</code> to make the widget enabled, 
    *          <code>FALSE</code> to disable the widget.
    */
   @Override
   public void setEnabled(boolean enabled)
   {
      // set flag for main part
      super.setEnabled(enabled);

      // set flag for possible entry filed
      if (entryField != null)
      {
         entryField.setEnabled(enabled);
      }

      // set flag for possible simple list
      if (simpleList != null)
      {
         simpleList.setEnabled(enabled);
      }

      // and post value changed event
      ThinClient.getInstance().independentEventDrawingBracket(ComboBoxGuiImpl.this, new Runnable()
      {
         @Override
         public void run()
         {
            repaint();
         }
      });
   }

   /**
    * Check if widget has focus.
    * 
    * @return  <code>true</code> if widget has focus.
    */
   @Override
   public boolean hasFocus()
   {
      // check the combo-box itself for focus
      boolean cbbFocused = super.hasFocus();
      
      return config().mode == ComboBoxConfig.Mode.SIMPLE &&
                simpleList != null && entryField != null
                   ? cbbFocused || simpleList.hasFocus() || entryField.hasFocus()
                   : config().mode == ComboBoxConfig.Mode.DROP_DOWN && entryField != null 
                      ? cbbFocused || entryField.hasFocus() || dropDownActive : cbbFocused;
   }
   
   /**
    * Destroy this widget.  Cleans up all including widgets before destruction.
    */
   public void destroy()
   {
      // deregister possible SIMPLE list components
      if (simpleListPane != null)
      {
         OutputManager.instance().getRegistry().removeContainer((Widget) simpleListPane);
      }

      // call superclass for main destruction
      super.destroy();
   }

   /**
    * Posts value changed event to the queue.  This should be done in GUI mode after any changes
    * in entry field for SIMLPLE and DROP_DOWN combo-box modes.
    */
   public void entryFieldChanged(String newValue)
   {
      // put an ActionEvent onto the system event queue
      EventManager.postEvent(new ActionEvent(this, newValue));
      
      // honor any Progress triggers
      KeyInput vchng = new KeyInput(this, EventType.KEY_PRESSED, Keyboard.SE_VALUE_CHANGED, true);
      EventManager.postEvent(vchng);
      
      ConfigHelper.setModified(config, true);
   }
   
   /**
    * Posts value changed event to the queue.  This should be done in ChUI and GUI mode after
    * closing drop-down or in GUI mode after pressing UP/DOWN keys with focused combo-box.
    */
   @Override
   protected void valueChangedCommon()
   {
      // set up entry field if exists
      if (entryField != null)
      {
         String newText = model().selected();
         // set new value
         entryField.setValue(new character(newText));
         // set up selection range to the full text selection
         entryField.config().selectionStart = 0;
         entryField.config().selectionEnd = newText.length();
      }
      
      // and call superclass
      super.valueChangedCommon();
   }
   
   /**
    * Exit the drop-down (the "selection" mode of the combo-box).  GUI specific cleanup.
    */
   @Override
   protected void exitDropDown()
   {
      // call generic cleanup from the superclass
      super.exitDropDown();

      // clean up registry from drop-down widgets added in GUI code
      OutputManager.instance().getRegistry().removeContainer((Widget) dropDown);
   }
   
   /**
    * Initializes the event set to return.  GUI specifics,
    * 
    * @return  The keyboard event to be used to exit drop-down.
    */
   @Override
   protected Set<String> initEventsSet()
   {
      // call common version
      Set<String> setGuiResult = super.initEventsSet();

      // add GUI specific messages
      setGuiResult.add("TAB");
      setGuiResult.add("LEAVE");
      
      return setGuiResult;
   }
   
   /**
    * Adding mouse clicks as exit events.  GUI specifics.
    * 
    * @param   listToChange
    *          The event list to modify.
    */
   @Override
   protected void addMouseExitEvent(EventList listToChange)
   {
      // call common version first
      super.addMouseExitEvent(listToChange);

      // mouse click on anywhere dismisses drop-down
      listToChange.addEvent("MOUSE-MENU-DOWN", true);
      listToChange.addEvent("MOUSE-EXTEND-DOWN", true);

      // enable drop-down painting
      ((DropDownGuiImpl)dropDown).enableDraw();
   }

   /**
    * Clean up the main registry pool from drop-down widgets when it is closing, if the widget is
    * a container, make recursive call.
    */
   private void createEntryField()
   {
      // get ID for new widget
      WidgetId efId = WidgetId.nextID();

      // create config for entry field, we use fill-in config for this widget,
      // meaning no server-side counterpart for this widget
      FillInConfig finCfg = new FillInConfig(efId);

      // inherit some options from combo-box config
      finCfg.labels = false;
      finCfg.frameId = config.frameId;
      finCfg.dataType = config.dataType;
      finCfg.setDynamicFormat(config.format);

      // create entry field
      entryField = new EntryFieldGuiImpl(finCfg, this);
      // this is required to properly initialize entry field
      GuiOutputManager.instance().getRegistry().addWidget(efId, entryField);
      // initialize entry field
      entryField.initialize(efId, finCfg);
      // add widget config to config manager
      ConfigManager.getInstance().addWidgetConfig(finCfg);

      // and set correct location
      NativePoint cbbLoc = physicalLocation();
      entryField.setPhysicalLocation(cbbLoc.x + ENTRY_SHIFT, cbbLoc.y + ENTRY_SHIFT);
      
      // enable and make visible
      entryField.setVisible(true);
      entryField.setEnabled(true);
      
      // add the widget to the frame the combo-box belongs to
      Frame<GuiOutputManager> frCombo = (Frame<GuiOutputManager>) UiUtils.locateFrame(this);
      frCombo.addWidget(entryField);
   }
   
   /**
    * Creates the list for SIMPLE mode of the combo-box.  This part will permanently exist for
    * the combo-box life time.
    */
   private void createSimpleList()
   {
      // create the drop-down box
      simpleList = screen().getFactory().createScrollableList(this, this.model());

      // register it as a new frame
      simpleListPane = screen().getFactory().createScrollPane(WidgetId.nextID(), simpleList);
      simpleListPane.setBorder(new EmptyBorder<GuiOutputManager>());

      // set height, width and position of the drop-down itself
      simpleList.setColumns(calcLimit(false));

      // set up visible rows, height and position
      int rows = config.innerLines;
      
      if (rows <= 0)
      {
         int numItems = model().size();
         
         // ensure that we always have at least 1 row
         rows = (numItems > 0) ? numItems : 1;
      }
      simpleList.setVisibleRows(rows);
      simpleListPane.setSize(simpleListPane.minimumSize());

      // set location in character units      
      Point origin = screenLocation();
      origin.y += 1;
      simpleListPane.setLocation(origin);
      
      // set up physical location just below the combo-box
      NativePoint slLoc = physicalLocation();
      // the shift to draw the list properly
      slLoc.x += 2;
      slLoc.y += physicalDimension().height + 4;
      simpleListPane.setPhysicalLocation(slLoc.x, slLoc.y);
      
      // force the initial selection
      int idx = model().calcSelection(config.items);
      
      simpleList.select(idx);
      // set up initial top row
      simpleList.setTopRow(idx);
     
      // set up notifications
      simpleList.addSelectionListener(this);

      // init scrollbars, visible on demand
      simpleListPane.setScrollBars(Bars.VERTICAL);
         
      ((ScrollableSelectionListGuiImpl)simpleListPane.getChild()).
                                       setScrollBar(simpleListPane.vbar());
                                       
      // enable and show the simple list
      simpleListPane._setVisible(true);
      simpleListPane.setEnabled(true);

      // adding to the combo-box parent frame
      Frame<GuiOutputManager> frCombo = (Frame<GuiOutputManager>) UiUtils.locateFrame(this);
      frCombo.addWidget(simpleListPane);

      // adding to the registry, no mouse event will be available if not to register
      OutputManager.instance().getRegistry().addContainer((Widget) simpleListPane);
   }

   /**
    * Changes the selected value without opening drop-down.
    * 
    * @param   indexNew
    *          The index of the new item to select.
    */
   private void setNewSelection(int indexNew)
   {
      // new value can not be out of the current range
      if (indexNew < 0 || indexNew >= model().size())
      {
         return;
      }
      
      // set up the model with new selection
      model().setSelectedIndex(indexNew);
      model().setSelectedId(indexNew + 1);
      model().select(model().get(indexNew));

      // and post value changed event
      ThinClient.getInstance().independentEventDrawingBracket(ComboBoxGuiImpl.this, new Runnable()
      {
         @Override
         public void run()
         {
            valueChangedCommon();
            repaint();
         }
      });
   }

   /**
    * Sets up initial image object for drop down button.
    * 
    * @param   name
    *          Image name for drop-down button.
    *
    * @return   Image object for the given name.
    */
   private ImageGuiImpl initDropDownButton(String name)
   {
      ImageGuiImpl imgResult = null;
      // get the image data for requested image
      byte[] imageData = ImageHelper.getPrepackagedImageData(name);
      // initialize the given image
      if (imageData != null)
      {
         imgResult = new ImageGuiImpl(WidgetId.nextID());
         imgResult.setImage(imageData);
         ((ImageConfig)imgResult.config()).transparent = true;
         // set up button location
         NativePoint location = imgResult.physicalLocation();
         // the size of the entry field
         NativeDimension dim = physicalDimension();
         // button is on the right corner of the entry field
         imgResult.setPhysicalLocation(location.x + dim.width - ButtonGuiImpl.PREBUILT_SIZE - 1,
                                       location.y + (dim.height - ButtonGuiImpl.PREBUILT_SIZE)/2);
      }
      return imgResult;
   }
   
   /**
    * Draw 3D border around different parts of the combo-box widget.
    * 
    * @param   orig
    *          Combo-box origin.
    * @param   dim
    *          Combo-box size.
    */
   private void drawFieldBorder(NativePoint orig, NativeDimension dim)
   {
      int x1 = orig.x;
      int x2 = orig.x + dim.width;
      int y1 = orig.y;
      int y2 = orig.y + dim.height;

      // drawing the frame around entry field
      gd.setColor(darkShadow);
      gd.drawLine(x1, y1, x1, y2);
      gd.drawLine(x1 + 1, y1, x2, y1);

      gd.setColor(gc.bgColor);
      gd.drawLine(x1 + 1, y2, x2, y2);
      gd.drawLine(x2, y2 - 1, x2, y1 + 1);

      gd.setColor(veryDarkShadow);
      gd.drawLine(x1 + 1, y1 + 1, x1 + 1, y2 - 1);
      gd.drawLine(x1 + 2, y1 + 1, x2 - 1, y1 + 1);
   }
   
   /**
    * Draw entry field of the combo-box widget.
    * 
    * @param   orig
    *          Combo-box origin.
    * @param   dim
    *          Combo-box size.
    * @param   fnt
    *          Font to use for combo-box entry field.
    * @param   focused
    *          Is the combo-box focused.
    */
   private void drawEntryField(NativePoint orig,
                               NativeDimension dim,
                               FontDetails<?> fnt,
                               boolean focused)
   {
      // coordinates of the field area
      int w = dim.width - ButtonGuiImpl.PREBUILT_SIZE - 4;
      int h = dim.height - 3; 

      // draw internal background
      gd.setColor(lightShadow);
      gd.drawRect(orig.x + 2, orig.y + 2, w, h );
      if (focused)
      {
         gd.setColor(selectionBack);
      }
      gd.fillRect(orig.x + 3, orig.y + 3, w - 1, h - 1);
      
      // get the text to draw with field
      String display = null;
      
      display = model().selected();
      
      if (display == null)
         display = new String("");

      // limit the data portion of the field to the maximum allowed size
      // in chui char coordinates are always whole numbers
      if (display.length() > (int) limit)
         display = display.substring(0, (int) limit);
      else if (display.length() < (int) limit)
         display = StringHelper.leftAlignText(display, (int) limit);

      // draw entry field text
      gd.setGuiFont(fnt);
      // left-center alignment
      int xt = orig.x + TEXT_SHIFT;
      int yt = orig.y + 1 + (dim.height + fnt.pointSize)/2;
      gd.setFontStyle(fnt.style);
      // enabled state controls text drawing
      if (isEnabled())
      {
         if (focused)
         {
            gd.setColor(lightShadow);
         }
         else
         {
            gd.setColor(gc.fgColor);
         }
         gd.drawString(display, xt, yt);
      }
      else
      {
         gd.setColor(lightShadow);
         gd.drawString(display, xt + 1, yt + 1);
         gd.setColor(darkShadow);
         gd.drawString(display, xt, yt);
      }

      // draw dotted rectangle around focused button text
      if (focused)
      {
         // set dotted line style
         gd.setLineStroke(LineStroke.DOTTED_SMALL);
         gd.setColor(selectionDots);
         gd.drawRect(orig.x + 3, orig.y + 3, w - 2, h - 2);
         // restore default line style
         gd.setLineStroke(LineStroke.DEFAULT);
      }
   }

   /**
    * Draw drop down button of the combo-box widget.
    * 
    * @param   orig
    *          Combo-box origin.
    * @param   dim
    *          Combo-box size.
    * @param   pressed
    *          Flag indicating the button state, pressed or released.
    */
   private void drawButton(NativePoint orig, NativeDimension dim, boolean pressed)
   {
      // simple check to prevent NPE
      if (imgButt == null || imgButtDisabled == null)
      {
         return;
      }
      
      // coordinates of the button area
      int x1 = orig.x + dim.width - ButtonGuiImpl.PREBUILT_SIZE - 1;
      int x2 = orig.x + dim.width;
      int y1 = orig.y + 2;
      int y2 = orig.y + dim.height;

      // draw the light frame around image
      gd.setColor(gc.bgColor);
      gd.fillRect(x1, y1, x2 - x1, y2 - y1);
      
      // disabled state has separate inage to draw  
      if (pressed)
      {
         // for pressed state need 1 pixel right-down shift 
         NativePoint location = imgButt.physicalLocation();
         imgButt.setPhysicalLocation(location.x + 1, location.y + 1);
         imgButt.draw();
         // draw small button frame
         gd.setColor(darkShadow);
         gd.drawRect(location.x, y1, x2 - x1 - 1, y2 - y1 - 1);
         // and restore origin after draw
         imgButt.setPhysicalLocation(location.x, location.y);
      }
      else
      {
         // determine what image to draw
         ImageGuiImpl imgReleased = null;
         if (isEnabled())
         {
            imgReleased = imgButt;
         }
         else
         {
            imgReleased = imgButtDisabled; 
         }
         imgReleased.draw();
         // draw small button frame
         NativePoint location = imgReleased.physicalLocation();
         gd.setColor(lightShadow);
         gd.drawLine(location.x + 1, y1 + 2,
                     location.x + 1, y2 - 3);
         gd.drawLine(location.x + 1, y1 + 1,
                     location.x + ButtonGuiImpl.PREBUILT_SIZE - 2, y1 + 1);

         gd.setColor(darkShadow);
         gd.drawLine(location.x + 1, y2 - 2,
                     location.x + ButtonGuiImpl.PREBUILT_SIZE - 1, y2 - 2);
         gd.drawLine(location.x + ButtonGuiImpl.PREBUILT_SIZE - 1, y1 + 1,
                     location.x + ButtonGuiImpl.PREBUILT_SIZE - 1, y2 - 3);

         gd.setColor(veryDarkShadow);
         gd.drawLine(location.x, y2 - 1,
                     location.x + ButtonGuiImpl.PREBUILT_SIZE, y2 - 1);
         gd.drawLine(location.x + ButtonGuiImpl.PREBUILT_SIZE, y1,
                     location.x + ButtonGuiImpl.PREBUILT_SIZE, y2 - 2);
      }
   }
}
